# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements the trigger of "ctrl+/".
"""

import re

shortcut = "ctrl+/"
sticky = False

class FindWord(object):

    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        self.document_manager = document_manager
    
    def activate(self):
        """
        Operations before trigger activation.
        """
        view = self.document_manager.get_active_view()
        buffer = view.buffer
        
        insert_mark = buffer.get_insert()
        begin = buffer.get_iter_at_mark(insert_mark)
        end = begin.copy()
        self.__get_word_bounds(begin, end)
        
        word = buffer.get_text(begin, end)
        
        search_bar = self.document_manager.interface.search_bar
        search_bar.show_bar()
        search_bar.entry.set_text(word)
        view.grab_focus()
        
        return True
        
    def __get_word_bounds(self, begin, end):
        pattern = re.compile("[a-z|A-Z|0-9]")
        
        # Find begin bound
        
        char = begin.get_char()
        if re.match(pattern, char):
            while True:
                if not(re.match(pattern, char)):
                    begin.forward_char()
                    break
                elif begin.starts_line():
                    break
                else:
                    begin.backward_char()
                char = begin.get_char()
                
        # Find end bound
        while True:
            char = end.get_char()
            if end.ends_line():
                break
            elif not(re.match(pattern, char)):
                break
            else:
                end.forward_char()
                    
        return (begin, end)
