# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements the trigger of "ctrl+shift+[".
"""

shortcut = "ctrl+shift+{"
sticky = False

class SelectInsideBrackets(object):

    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        self.document_manager = document_manager
    
    def activate(self):
        """
        Trigger activator.
        """
        view = self.document_manager.get_active_view()
        buffer = view.buffer
        open_brackets = {'(' : ')', '[' : ']', '{' : '}'}
        close_brackets = {')' : '(', ']' : '[', '}' : '{'}
        
        insert_mark = buffer.get_insert()
        begin = buffer.get_iter_at_mark(insert_mark)
        end = begin.copy()
        end.backward_char()
        
        left = None
        right = None
        
        while True:
            if left == None:
                begin.backward_char()
            if right == None:
                end.forward_char()
            
            begin_char = begin.get_char()
            end_char = end.get_char()
            
            if right == None:
                if begin_char in open_brackets.keys():
                    left = begin_char
            else:
                if begin_char == close_brackets[right]:
                    left = begin_char
	                
            if left == None:
                if end_char in close_brackets.keys():
                    right = end_char
            else:
                if end_char == open_brackets[left]:
                    right = end_char
        
            if left and right:
                break
        
        begin.forward_char()
        buffer.select_range(end, begin)
        
        return True
        
    def __search_close_match(self, iterator, char, match):
        iter = iterator.copy()
        cont = 1
        
        while iter.forward_char():
            c = iter.get_char()
            if c == char:
                cont+=1
            elif c == match:
                cont-=1
            
            if cont == 0:
                return iter
        
        return iterator
        
    def __search_open_match(self, iterator, char, match):
        iter = iterator.copy()
        cont = 1
        
        while iter.backward_char():
            c = iter.get_char()
            if c == char:
                cont+=1
            elif c == match:
                cont-=1
            
            if cont == 0:
                return iter
        
        return iterator

