# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements the trigger of "tab".
"""

import re

shortcut = u'\uff09'
sticky = False

class Tab(object):
    
    """
    This class implements all functions of tab shortcut.
    """
    
    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        
        self.document_manager = document_manager
        self.snippet_flag = False
         
    def activate(self):
        """
        Operations after trigger activation.
        """
        view = self.document_manager.get_active_view()
        buffer = view.buffer
        snippets = view.snippets
        
#        insert_mark = buffer.get_insert()
#        insert_iter = buffer.get_iter_at_mark(insert_mark)

#        start_iter = insert_iter.copy()

#        self.__find_word_start(start_iter)
#                
#        start_del_mark = buffer.create_mark(None, start_iter, True)
#        end_del_mark = buffer.create_mark(None, insert_iter, True)
#        
#        key = buffer.get_text(start_iter, insert_iter)
        
#        try:
#            snippets.active()
#            self.snippet_flag = True
#        except KeyError:
#            self.snippet_flag = False
#            if len(snippets.stack) == 0:
#                return False
#        except AttributeError:
#            self.snippet_flag = False
#            if len(snippets.stack) == 0:
#                return False
#        else:
#            si = buffer.get_iter_at_mark(start_del_mark)
#            ei = buffer.get_iter_at_mark(end_del_mark)
#            buffer.delete(si, ei)
        self.snippet_flag = snippets.active()
        
        if not self.snippet_flag:
            if len(snippets.stack) > 0:
                if snippets.valid_cursor_position():
                    for i in range(len(snippets.stack)):
                        try:
                            snippets.next_field()
                        except KeyError:
                            end_mark = snippets.stack[-1].stop
                            
                            if end_mark != None:
                                buffer.place_cursor(buffer.get_iter_at_mark(end_mark))
                            else:
                                end_bound_mark = snippets.stack[-1].bounds[1]
                                buffer.place_cursor(buffer.get_iter_at_mark(end_bound_mark))
#                                buffer.insert_at_cursor("\t")
                                
                            snippets.pop_snippet()
                        else:
                            break
                else:
                    snippets.pop_snippet()
#                    buffer.insert_at_cursor("\t")
                    return False
                return True
            else:
                return False
        else:
            return True
#        self.snippet_flag = False
#        
#        return True
        
    def __find_word_start(self, iterator):
        pattern = re.compile("[a-z|A-Z|0-9]")
        symbols = ('!','@','#', '$', '%', '&', '*',
                   '(', ')', '-' ,'+', '.', ',', '~', '^')
        iterator.backward_char()
        if iterator.get_char() in symbols:
            return
        while True:
            char = iterator.get_char()
            if not(re.match(pattern, char)):
                iterator.forward_char()
                return
            elif iterator.starts_line():
                return
            else:
                iterator.backward_char()

